var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import debug from 'debug';
import assert from 'assert';
import { peerIdFromString } from '@libp2p/peer-id';
import { DEFAULT_PING_INTERVAL } from './constants.js';
const log = debug('libp2p:peers-latency-tracker');
export class PeersLatencyTracker {
    constructor(node, options) {
        var _a;
        this._peersLatencyMap = new Map();
        this._node = node;
        this._pingInterval = (_a = options.pingInterval) !== null && _a !== void 0 ? _a : DEFAULT_PING_INTERVAL;
        this._setPingMethod();
        if (!this._ping) {
            log('Latency tracker not enabled as Ping service is not configured in libp2p');
            return;
        }
        this._addConnectionListeners();
        // Create map from already connected peers
        this._peersLatencyMap = new Map(this._node
            .getPeers()
            .map(peerId => [peerId.toString(), []]));
        this._intervalID = this._startLatencyTracker();
    }
    get enabled() {
        return Boolean(this._ping);
    }
    getLatencyValues(peerIdString) {
        return this._peersLatencyMap.get(peerIdString);
    }
    destroy() {
        clearInterval(this._intervalID);
        this._node.removeEventListener('peer:connect', this._handlePeerConnect);
        this._node.removeEventListener('peer:disconnect', this._handlePeerDisconnect);
    }
    // Set ping method for old or new version of libp2p
    _setPingMethod() {
        // Check if ping method exists in libp2p instance for versions < 0.45.0
        if ('ping' in this._node) {
            this._ping = this._node.ping.bind(this._node);
            return;
        }
        // https://github.com/libp2p/js-libp2p/blob/master/doc/migrations/v0.44-v0.45.md#services
        // eslint-disable-next-line dot-notation
        const services = this._node['services'];
        // For libp2p versions > 0.45.0 check if Ping service is configured in libp2p instance
        if (services.ping) {
            const pingService = services.ping;
            this._ping = pingService.ping.bind(pingService);
        }
    }
    _addConnectionListeners() {
        this._node.addEventListener('peer:connect', this._handlePeerConnect.bind(this));
        this._node.addEventListener('peer:disconnect', this._handlePeerDisconnect.bind(this));
    }
    // https://github.com/libp2p/js-libp2p/blob/master/doc/migrations/v0.44-v0.45.md#peerconnect
    _handlePeerConnect(event) {
        const peerIdString = this._getPeerIdFromConnectionEvent(event).toString();
        if (!this._peersLatencyMap.has(peerIdString)) {
            // Add peer to map if it already doesn't exist
            this._peersLatencyMap.set(peerIdString, []);
        }
    }
    // https://github.com/libp2p/js-libp2p/blob/master/doc/migrations/v0.44-v0.45.md#peerdisconnect
    _handlePeerDisconnect(event) {
        const remotePeer = this._getPeerIdFromConnectionEvent(event);
        // Remove peer as peer:disconnect event is triggered when there are no connections to a peer
        this._peersLatencyMap.delete(remotePeer.toString());
    }
    // https://github.com/libp2p/js-libp2p/blob/master/doc/migrations/v0.44-v0.45.md#event-changes
    _getPeerIdFromConnectionEvent({ detail }) {
        if ('remotePeer' in detail) {
            return detail.remotePeer;
        }
        return detail;
    }
    _startLatencyTracker() {
        return setInterval(() => {
            this._updatePeersLatency();
        }, this._pingInterval);
    }
    _updatePeersLatency() {
        // Ping and update latency for each peer asynchronously
        this._peersLatencyMap.forEach((latencyValues, peerIdString) => __awaiter(this, void 0, void 0, function* () {
            try {
                // Ping remote peer
                const peerId = peerIdFromString(peerIdString);
                assert(this._ping);
                const latency = yield this._ping(peerId);
                // Update latency values with latest
                const length = latencyValues.unshift(latency);
                if (length > 5) {
                    // Pop oldest latency value from list
                    latencyValues.pop();
                }
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
            }
            catch (err) {
                // On error i.e. no pong
                log(err === null || err === void 0 ? void 0 : err.message);
            }
        }));
    }
}
//# sourceMappingURL=peers-latency-tracker.js.map